Package: CDSim
Type: Package
Title: Simulating Climate Data for Research and Modelling
Version: 0.1.2
Date: 2026-04-19
Authors@R: 
    c(
    person("Isaac", "Osei", role = c("aut", "cre"), email = "ikemillar65@gmail.com"),
    person("Acheampong", "Baafi-Adomako", role = "aut", email = "acheampongbaafi1@gmail.com")
    )
Maintainer: Isaac Osei <ikemillar65@gmail.com>
Description: Generate synthetic station-based monthly climate time-series including
    temperature and rainfall, export to Network Common Data Form (NetCDF), 
    and provide visualization helpers for climate workflows. The approach is 
    inspired by statistical weather generator concepts described in Wilks (1999)
    <doi:10.1016/S0168-1923(99)00037-4> and Richardson (1981) <doi:10.1029/WR017i001p00182>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: trend, truncnorm, ncdf4, lubridate, readr, dplyr, ggplot2,
        rlang, tidyr, vroom, tibble, stats
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
URL: https://github.com/ikemillar/CDSim
BugReports: https://github.com/ikemillar/CDSim/issues
NeedsCompilation: no
Packaged: 2026-04-22 13:38:18 UTC; isaacosei
Author: Isaac Osei [aut, cre],
  Acheampong Baafi-Adomako [aut]
Repository: CRAN
Date/Publication: 2026-04-22 14:10:02 UTC
Built: R 4.6.0; ; 2026-04-29 13:38:02 UTC; unix

# Generated by roxygen2: do not edit by hand

export(create_stations)
export(plot_station_timeseries)
export(safe_name)
export(simulate_climate_series)
export(validate_climate)
export(validate_climate_internal)
export(write_station_csv)
export(write_station_netcdf)
importFrom(dplyr,`%>%`)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_x_date)
importFrom(ggplot2,theme_dark)
importFrom(ggplot2,theme_minimal)
importFrom(lubridate,days_in_month)
importFrom(lubridate,month)
importFrom(ncdf4,nc_close)
importFrom(ncdf4,nc_create)
importFrom(ncdf4,ncatt_put)
importFrom(ncdf4,ncdim_def)
importFrom(ncdf4,ncvar_def)
importFrom(ncdf4,ncvar_put)
importFrom(readr,read_csv)
importFrom(rlang,sym)
importFrom(stats,acf)
importFrom(stats,arima.sim)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,rbinom)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,wilcox.test)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_wider)
importFrom(truncnorm,rtruncnorm)
importFrom(vroom,vroom_write)

---
title: "README"
output: html_document
---

# CDSim

Simulate station-based monthly climate data (Tmin/Tmax/rainfall), export station NetCDF, and provide plotting helpers.

## Quick start

```r
# install devtools then load package folder locally
# devtools::load_all("path/to/CDSim")

library(CDSim)
stations <- create_stations(n = 11, seed = 42)
sim <- simulate_climate_series(stations, 1981, 1985, seed = 101)
plot_station_timeseries(sim, "Station_1", "Avg.Tx")
write_station_netcdf(sim, "sim_example.nc")

#' Create or load station metadata
#'
#' Create a station metadata table (Station, LON, LAT) either by:
#' - loading from a CSV file,
#' - accepting an existing data.frame,
#' - or auto-generating synthetic stations in a bounding box.
#'
#' @param source Path to CSV file OR a data.frame with Station/LON/LAT OR NULL (to generate synthetic).
#' @param n Integer number of stations to generate when source = NULL. Default 10.
#' @param bbox numeric vector c(min_lon, max_lon, min_lat, max_lat). Default ~ Ghana bounding box.
#' @param seed Optional numeric to make generation reproducible.
#' @return A data.frame with columns Station, LON, LAT.
#' @examples
#' create_stations(n = 5, seed = 42)
#' create_stations(data.frame(Station="A", LON=0, LAT=5))
#' @export
#' @importFrom stats runif
#' @importFrom readr read_csv

create_stations <- function(source = NULL, n = 10, bbox = c(-3.5, 1.5, 4.5, 11.5), seed = NULL) {
  if (!is.null(seed)) set.seed(seed)

  if (is.character(source)) {
    if (!file.exists(source)) stop("File not found: ", source)
    message("Loading station list from file: ", source)
    df <- readr::read_csv(source, show_col_types = FALSE)
  } else if (is.data.frame(source)) {
    message("Using station metadata from supplied data.frame")
    df <- source
  } else if (is.null(source)) {
    message("Generating synthetic station network...")
    if (length(bbox) != 4 || !is.numeric(bbox)) {
      stop("bbox must be numeric vector: c(min_lon, max_lon, min_lat, max_lat)")
    }
    df <- data.frame(
      Station = paste0("Station_", seq_len(n)),
      LON = runif(n, bbox[1], bbox[2]),
      LAT = runif(n, bbox[3], bbox[4]),
      stringsAsFactors = FALSE
    )
    message("Generated ", n, " synthetic stations within bounding box.")
  } else {
    stop("`source` must be a file path, a data.frame, or NULL.")
  }

  # ensure required cols exist, try to coerce common alternatives
  names(df) <- make.names(names(df))
  if (!all(c("Station", "LON", "LAT") %in% names(df))) {
    stop("Station metadata must contain columns named: Station, LON, LAT")
  }
  df$Station <- as.character(df$Station)
  df$LON <- as.numeric(df$LON)
  df$LAT <- as.numeric(df$LAT)
  rownames(df) <- NULL
  return(df[, c("Station", "LON", "LAT")])
}

test_that("simulate_climate_series produces expected columns", {
  st <- create_stations(n = 2, seed = 1)
  sim <- simulate_climate_series(st)

  expect_true(all(c("Station", "Date", "Avg.Tn") %in% names(sim)))
  expect_gt(nrow(sim), 0)
})

#' Make a safe filename
#'
#' Ensures file names contain only safe ASCII characters.
#'
#' @param x A character string to clean.
#'
#' @return A cleaned filename string.
#' @export
safe_name <- function(x) {
  gsub("[^A-Za-z0-9_]", "_", x)
}

#' Visualization Functions for Climate Data
#'
#' @name visualization
NULL
#' Plot Station Time Series with Seasonal Detection
#'
#' @description Creates a time-series plot for climate variables with automatic hemisphere-based season detection.
#'
#' @param df A tidy dataset containing columns: `Station`, `Date`, `LAT`, and variables.
#' @param station Station name.
#' @param var Climate variable to plot.
#' @param smooth Add LOESS smoothing line.
#' @param theme_dark Use dark theme.
#'
#' @return A ggplot object.
#'
#' @examples
#' stations <- create_stations(n = 3)
#' sim <- simulate_climate_series(stations)
#' plot_station_timeseries(sim, station = "Station_1", var = "Avg.Tn")
#'
#' @importFrom ggplot2 ggplot aes geom_line geom_point geom_smooth labs scale_x_date scale_color_manual theme_minimal theme_dark element_text
#' @importFrom dplyr filter mutate case_when `%>%`
#' @importFrom lubridate month
#' @importFrom stats median rnorm rgamma
#' @export
plot_station_timeseries <- function(df, station, var = "Avg.Tn", smooth = TRUE, theme_dark = FALSE) {
  stopifnot(var %in% c("Avg.Tn","Avg.Tx","Sum.Rf","Avg.Rf"))
  dsub <- df %>% dplyr::filter(Station == station)
  # Determine hemisphere via median latitude
  hemi <- if (abs(median(dsub$LAT)) < 5) {
    "Equatorial"
  } else if (median(dsub$LAT) > 0) {
    "Northern"
  } else {
    "Southern"
  }
  # Add month column
  dsub <- dsub %>% dplyr::mutate(Month = lubridate::month(Date))
  # Assign seasons based on region
  dsub <- dsub %>%
    dplyr::mutate(
      Season = dplyr::case_when(
        # Equatorial tropical pattern
        hemi == "Equatorial" & Month %in% c(11,12,1,2,3) ~ "Dry Season",
        hemi == "Equatorial" ~ "Wet Season",
        # Northern Hemisphere
        hemi == "Northern" & Month %in% c(12,1,2) ~ "Winter",
        hemi == "Northern" & Month %in% c(3,4,5) ~ "Spring",
        hemi == "Northern" & Month %in% c(6,7,8) ~ "Summer",
        hemi == "Northern" & Month %in% c(9,10,11) ~ "Autumn",
        # Southern Hemisphere (inverted)
        hemi == "Southern" & Month %in% c(6,7,8) ~ "Winter",
        hemi == "Southern" & Month %in% c(9,10,11) ~ "Spring",
        hemi == "Southern" & Month %in% c(12,1,2) ~ "Summer",
        hemi == "Southern" & Month %in% c(3,4,5) ~ "Autumn",
        TRUE ~ "Unknown"
      )
    )

  # Color theme per variable
  var_colors <- list(
    "Avg.Tn" = list(line = "#3182bd", smooth = "#08519c"),
    "Avg.Tx" = list(line = "#e31a1c", smooth = "#a50f15"),
    "Sum.Rf" = list(line = "#084594", smooth = "#08306b"),
    "Avg.Rf" = list(line = "#2ca25f", smooth = "#006d2c")
  )
  # Universal seasonal palette
  season_palette <- c(
    "Winter" = "#81b1d2",
    "Spring" = "#4daf4a",
    "Summer" = "#e41a1c",
    "Autumn" = "#ff7f00",
    "Wet Season" = "#1b9e77",
    "Dry Season" = "#d95f02",
    "Unknown" = "grey60"
  )
  p <- ggplot2::ggplot(dsub, ggplot2::aes(x = Date, y = !!rlang::sym(var))) +
    ggplot2::geom_line(linewidth = 1, color = var_colors[[var]]$line) +
    ggplot2::geom_point(aes(color = Season), size = 2) +
    ggplot2::scale_color_manual(values = season_palette) +
    ggplot2::scale_x_date(date_breaks = "2 years", date_labels = "%Y") +
    ggplot2::labs(
      title = paste(var, "Time Series at", station),
      subtitle = paste("Hemisphere:", hemi,
                       " | Period:", format(min(dsub$Date), "%Y"), "-", format(max(dsub$Date), "%Y")),
      x = "Date",
      y = var,
      color = "Season"
    ) +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold", size = 16),
      axis.text.x = ggplot2::element_text(angle = 45, vjust = 0.5),
      legend.position = "bottom"
    )
  if (smooth) {
    p <- p + ggplot2::geom_smooth(
      method = "loess",
      span = 0.25,
      linewidth = 0.9,
      color = var_colors[[var]]$smooth,
      alpha = 0.25
    )
  }
  if (theme_dark) {
    p <- p + ggplot2::theme_dark()
  }
  return(p)
}
utils::globalVariables(c("Season"))

#' CDSim: Climate Data Simulation Toolkit
#'
#' Tools for generating and exporting synthetic climate observation datasets.
#'
#' @docType package
#' @author Isaac Osei and Acheampong Baafi-Adomako and Sivaparvathi Dusari
#' @keywords climate rainfall temperature simulation netcdf
"_PACKAGE"

#' Simulate monthly climate time series for stations
#'
#' Simulate monthly Tmin, Tmax, monthly total rainfall (Sum.Rf) and mean daily rainfall (Avg.Rf)
#' for each station across a year range.
#'
#' @param stations data.frame from create_stations() (Station, LON, LAT)
#' @param start_year integer (e.g., 1996)
#' @param end_year integer (e.g., 2025)
#' @param seed optional numeric seed
#' @param temp_trend_per_year temperature trend per year (°C/year warming)
#' @param rain_trend_per_year rain trend per year (slight drying trend)
#' @param phi_temp AR(1) persistence
#' @param sd standard deviation of the AR(1) innovation process controlling temperature variability
#' @param Tmin_min minimum value for minimum temperature
#' @param Tmin_max maximum value for minimum temperature
#' @param Tmax_min minimum value for maximum temperature
#' @param Tmax_max maximum value for maximum temperature
#' @return A tidy data.frame with one row per station × month containing:
#'   Station, LON, LAT, Year, Month, Date, Avg.Tn, Avg.Tx, Sum.Rf, Avg.Rf
#' @seealso [write_station_csv()], [write_station_netcdf()]
#' @family IO Functions
#' @details
#' This function generates synthetic monthly climate time series using a
#' stochastic, physically-informed modelling framework. Temperature is modeled
#' as a combination of deterministic seasonality, long-term trend, and
#' stochastic variability. The seasonal component is represented using a
#' sinusoidal function, while temporal persistence is introduced via an
#' autoregressive AR(1) process applied to the innovation term.
#'
#' Minimum temperature (Avg.Tn) is simulated using a truncated normal
#' distribution to enforce physically realistic lower and upper bounds.
#' Maximum temperature (Avg.Tx) is generated using a gamma-distributed
#' perturbation applied to the mean temperature, producing an asymmetric
#' distribution consistent with observed climatological behavior.
#'
#' Rainfall occurrence is modeled using a first-order Markov chain, allowing
#' for realistic wet–dry persistence. Conditional on occurrence, rainfall
#' intensity is drawn from a gamma distribution with seasonally varying mean.
#' A temporal trend term can be applied to represent long-term climatic changes
#' such as gradual drying or wetting.
#'
#' To ensure physical consistency between variables, a coupling mechanism is
#' introduced whereby increased rainfall (proxy for cloud cover) reduces
#' maximum temperature through a linear cooling adjustment. This enforces
#' a negative dependence between precipitation and temperature consistent
#' with atmospheric energy balance principles.
#'
#' Finally, a minimum diurnal temperature difference constraint is enforced
#' after rounding to guarantee that Avg.Tx > Avg.Tn at all time steps, while
#' preserving the statistical distribution of the simulated variables.
#'
#' The default parameterization reflects typical tropical conditions for Ghana,
#' but all parameters are user-configurable, allowing adaptation to other
#' climatic regions.
#' The modelling approach follows established stochastic weather generation
#' principles while extending them with distributional asymmetry and
#' cross-variable coupling for improved physical realism.
#' @examples
#' st <- create_stations(n = 3, seed = 1)
#' sim <- simulate_climate_series(st, 1996, 2025, seed = 42)
#' head(sim)
#' @export
#' @importFrom truncnorm rtruncnorm
#' @importFrom lubridate days_in_month
#' @importFrom stats arima.sim
simulate_climate_series <- function(
    stations,
    start_year = 1996,
    end_year = 2025,
    seed = NULL,
    temp_trend_per_year = 0.02,    # °C/year warming
    rain_trend_per_year = -0.003,  # slight drying trend
    phi_temp = 0.85,               # AR(1) persistence
    sd = 0.4,                      # standard deviation for AR(1)
    # --- Temperature bounds (Ghana defaults) ---
    Tmin_min = 18,
    Tmin_max = 30,
    Tmax_min = 24,
    Tmax_max = 42
) {
  if (!is.null(seed)) set.seed(seed)
  # --- validation checks ---
  stopifnot(is.data.frame(stations))
  stopifnot(all(c("Station", "LON", "LAT") %in% names(stations)))
  if (Tmin_min >= Tmin_max) {
    stop("Tmin_min must be less than Tmin_max")
  }
  if (Tmax_min >= Tmax_max) {
    stop("Tmax_min must be less than Tmax_max")
  }
  dates <- seq(as.Date(sprintf("%04d-01-15", start_year)),
               as.Date(sprintf("%04d-12-15", end_year)),
               by = "month")
  n <- length(dates)
  monthly <- as.integer(format(dates, "%m"))
  years <- as.numeric(format(dates, "%Y"))
  generate_ar1 <- function(n, phi, sigma) {
    as.numeric(arima.sim(n = n, list(ar = phi), sd = sigma))
  }
  out <- lapply(seq_len(nrow(stations)), function(i) {
    s <- stations[i, , drop = FALSE]
    # --- Seasonal structure ---
    base_mean <- 26 - 0.08 * s$LAT
    amp <- 4 + 0.03 * abs(s$LAT - mean(stations$LAT))
    seasonal <- amp * sin(2 * pi * (monthly - 1) / 12)
    trend <- temp_trend_per_year * (years - start_year)
    noise <- generate_ar1(n, phi_temp, sd)
    tmean <- base_mean + seasonal + trend + noise
    # --- Tmin (bounded truncated normal) ---
    Tmin <- truncnorm::rtruncnorm(
      n,
      a = Tmin_min,
      b = Tmin_max,
      mean = tmean - 5.5,
      sd = 1.2
    )
    # --- Tmax (gamma + bounds) ---
    Tmax_raw <- tmean + rgamma(n, shape = 4, scale = 1.2)
    # --- Rainfall occurrence (Markov chain) ---
    wet <- numeric(n)
    p_wet <- ifelse(monthly %in% 4:10, 0.65, 0.25)
    wet[1] <- rbinom(1, 1, p_wet[1])
    for (t in 2:n) {
      p <- if (wet[t - 1] == 1) 0.75 else 0.30
      wet[t] <- rbinom(1, 1, p)
    }
    # --- Rainfall intensity (fixed phase shift) ---
    r_mean <- 70 * (1 + 0.3 * sin(2 * pi * (monthly - 7)/12))
    rain_trend <- 1 + rain_trend_per_year * (years - start_year)
    r_mean <- pmax(5, r_mean * rain_trend)
    SumRf <- ifelse(
      wet == 1,
      rgamma(n, shape = 2.5, scale = r_mean / 2.5),
      0
    )
    # --- Rainfall–temperature coupling ---
    cloud_index <- as.numeric(scale(SumRf))
    cloud_index[is.na(cloud_index)] <- 0
    Tmax <- Tmax_raw - 1.5 * cloud_index
    # --- Enforce bounds smoothly ---
    Tmin <- pmin(pmax(Tmin, Tmin_min), Tmin_max)
    Tmax <- pmin(pmax(Tmax, Tmax_min), Tmax_max)
    idx <- Tmax >= Tmax_max
    if (any(idx)) {
      Tmax[idx] <- Tmax_max - runif(sum(idx), 0, 0.3)
    }
    Tmin <- round(Tmin, 1)
    # ensure Tmax > Tmin
    idx <- Tmax <= Tmin
    if (any(idx)) {
      Tmax[idx] <- Tmin[idx] + 0.5
    }
    Tmax <- round(Tmax, 1)
    AvgRf <- SumRf / lubridate::days_in_month(dates)
    data.frame(
      Station = s$Station,
      LON = s$LON,
      LAT = s$LAT,
      Year = years,
      Month = monthly,
      Date = dates,
      Avg.Tn = Tmin,
      Avg.Tx = Tmax,
      Sum.Rf = round(SumRf, 1),
      Avg.Rf = round(AvgRf, 2),
      stringsAsFactors = FALSE
    )
  })
  simdf <- do.call(rbind, out)
  rownames(simdf) <- NULL
  return(simdf)
}

#' Validate simulated climate data against observations
#'
#' Performs statistical and physical validation of simulated climate data
#' against observed datasets, including distributional tests, mean comparison,
#' dependence structure, and temporal persistence.
#'
#' @param sim Simulated climate data.frame
#' @param obs Observed climate data.frame
#'
#' @return A list containing validation metrics and test results
#' @export
#' @importFrom stats rbinom ks.test t.test wilcox.test cor acf sd lm coef
validate_climate <- function(sim, obs) {
  stopifnot(is.data.frame(sim), is.data.frame(obs))
  required_cols <- c("Avg.Tx", "Avg.Tn", "Sum.Rf")
  stopifnot(all(required_cols %in% names(sim)))
  stopifnot(all(required_cols %in% names(obs)))
  # --- Helper function for safe tests ---
  safe_test <- function(expr) {
    tryCatch(expr, error = function(e) NA)
  }
  # --- Distribution tests (K-S) ---
  ks_tests <- list(
    Tmax = safe_test(ks.test(sim$Avg.Tx, obs$Avg.Tx)),
    Tmin = safe_test(ks.test(sim$Avg.Tn, obs$Avg.Tn)),
    Rain = safe_test(ks.test(sim$Sum.Rf, obs$Sum.Rf))
  )
  # --- Mean comparison ---
  mean_tests <- list(
    ttest_Tmax = safe_test(t.test(sim$Avg.Tx, obs$Avg.Tx)),
    ttest_Tmin = safe_test(t.test(sim$Avg.Tn, obs$Avg.Tn)),
    ttest_Rain = safe_test(t.test(sim$Sum.Rf, obs$Sum.Rf)),

    wilcox_Tmax = safe_test(wilcox.test(sim$Avg.Tx, obs$Avg.Tx)),
    wilcox_Tmin = safe_test(wilcox.test(sim$Avg.Tn, obs$Avg.Tn)),
    wilcox_Rain = safe_test(wilcox.test(sim$Sum.Rf, obs$Sum.Rf))
  )

  # --- Correlation structure (physical coupling) ---
  correlation <- list(
    sim = cor(sim[, c("Sum.Rf", "Avg.Tx", "Avg.Tn")], use = "complete.obs"),
    obs = cor(obs[, c("Sum.Rf", "Avg.Tx", "Avg.Tn")], use = "complete.obs")
  )

  # --- Autocorrelation (lag-1) ---
  acf_sim <- list(
    Tmax = acf(sim$Avg.Tx, plot = FALSE)$acf[2],
    Tmin = acf(sim$Avg.Tn, plot = FALSE)$acf[2],
    Rain = acf(sim$Sum.Rf, plot = FALSE)$acf[2]
  )
  acf_obs <- list(
    Tmax = acf(obs$Avg.Tx, plot = FALSE)$acf[2],
    Tmin = acf(obs$Avg.Tn, plot = FALSE)$acf[2],
    Rain = acf(obs$Sum.Rf, plot = FALSE)$acf[2]
  )

  # --- Trend analysis (Mann-Kendall) ---
  if (requireNamespace("trend", quietly = TRUE)) {
    trend_tests <- list(
      Tmax = trend::mk.test(sim$Avg.Tx),
      Rain = trend::mk.test(sim$Sum.Rf)
    )
  } else {
    trend_tests <- NA
  }
  # --- Summary metrics ---
  summary_stats <- data.frame(
    Variable = c("Tmax", "Tmin", "Rain"),
    Mean_sim = c(mean(sim$Avg.Tx), mean(sim$Avg.Tn), mean(sim$Sum.Rf)),
    Mean_obs = c(mean(obs$Avg.Tx), mean(obs$Avg.Tn), mean(obs$Sum.Rf)),
    SD_sim = c(sd(sim$Avg.Tx), sd(sim$Avg.Tn), sd(sim$Sum.Rf)),
    SD_obs = c(sd(obs$Avg.Tx), sd(obs$Avg.Tn), sd(obs$Sum.Rf))
  )
  return(list(
    ks_tests = ks_tests,
    mean_tests = mean_tests,
    correlation = correlation,
    autocorrelation = list(sim = acf_sim, obs = acf_obs),
    trend = trend_tests,
    summary = summary_stats
  ))
}


#' Internal validation of simulated climate data
#'
#' Evaluates physical plausibility and statistical properties of simulated
#' climate data in the absence of observational datasets. The function
#' assesses distributional characteristics, temporal persistence,
#' inter-variable relationships, and physical constraints.
#'
#' @param sim Simulated climate data.frame
#'
#' @return A list of validation diagnostics
#' @export
#' @importFrom stats cor acf sd lm coef
validate_climate_internal <- function(sim) {
  stopifnot(is.data.frame(sim))
  required_cols <- c("Avg.Tx", "Avg.Tn", "Sum.Rf")
  stopifnot(all(required_cols %in% names(sim)))
  # Remove NA safely
  sim <- sim[complete.cases(sim[, required_cols]), ]
  # --- Physical constraints ---
  checks <- list(
    Tmin_min = min(sim$Avg.Tn),
    Tmax_max = max(sim$Avg.Tx),
    Rain_min = min(sim$Sum.Rf),
    Tmax_gt_Tmin = all(sim$Avg.Tx > sim$Avg.Tn),
    Tmin_plausible = min(sim$Avg.Tn) >= 18
  )

  # --- Distribution properties ---
  safe_sd <- function(x) ifelse(sd(x) == 0, NA, sd(x))
  skewness <- function(x) {
    m <- mean(x)
    s <- safe_sd(x)
    if (is.na(s)) return(NA)
    mean((x - m)^3) / s^3
  }

  distribution <- list(
    rain_skewness = skewness(sim$Sum.Rf),
    Tmax_skewness = skewness(sim$Avg.Tx),
    Tmin_skewness = skewness(sim$Avg.Tn),
    sd_Tmax = sd(sim$Avg.Tx),
    sd_Tmin = sd(sim$Avg.Tn),
    sd_Rain = sd(sim$Sum.Rf)
  )

  # --- Correlation (physical coupling) ---
  correlation <- cor(sim[, c("Sum.Rf", "Avg.Tx", "Avg.Tn")],
                     use = "complete.obs")
  rain_temp_relationship <- correlation["Sum.Rf", "Avg.Tx"] < 0
  # --- Autocorrelation ---
  acf_vals <- list(
    Tmax = acf(sim$Avg.Tx, plot = FALSE)$acf[2],
    Tmin = acf(sim$Avg.Tn, plot = FALSE)$acf[2],
    Rain = acf(sim$Sum.Rf, plot = FALSE)$acf[2]
  )
  # --- Simple trend check (linear approximation) ---
  time_index <- seq_along(sim$Avg.Tx)
  trend <- list(
    Tmax_slope = coef(lm(sim$Avg.Tx ~ time_index))[2],
    Rain_slope = coef(lm(sim$Sum.Rf ~ time_index))[2]
  )
  # --- Seasonal pattern check ---
  seasonal_means <- tapply(sim$Sum.Rf, sim$Month, mean)
  seasonality <- list(
    peak_month = which.max(seasonal_means),
    trough_month = which.min(seasonal_means)
  )
  overall_valid <- all(
    checks$Tmax_gt_Tmin,
    checks$Tmin_plausible,
    rain_temp_relationship,
    acf_vals$Tmax > 0.5
  )
  return(list(
    summary = summary(sim),
    checks = checks,
    distribution = distribution,
    correlation = correlation,
    rain_temp_coupling = rain_temp_relationship,
    autocorrelation = acf_vals,
    trend = trend,
    seasonality = seasonality,
    valid = overall_valid
  ))
}

#' Write station CSV
#' Exports a simulated climate station dataset to a CSV file.
#' @param df A dataframe returned by `simulate_climate_series()`.
#' @param file The output CSV filename.
#' @return Returns the file path invisibly.
#' @export
#' @examples
#' stations <- create_stations(n = 3)
#' sim <- simulate_climate_series(stations)
#' tmp <- tempfile(fileext = ".csv")
#' write_station_csv(sim, tmp)
#'
#' @importFrom vroom vroom_write
write_station_csv <- function(df, file = "simulated_station_climate.csv") {
  stopifnot(is.data.frame(df))  # safety check
  vroom::vroom_write(
    x = df,
    file = file,
    delim = ","
  )
  invisible(file)
}

#'
#' Write station NetCDF (station x time)
#' Exports a simulated climate station dataset to a NetCDF file.
#' @param df station x time long dataframe returned by simulate_climate_series()
#' @param out_nc Output NetCDF filename
#' @param fillvalue Value used for missing entries
#' @return Returns the file path invisibly.
#' @export
#' @importFrom dplyr arrange distinct select slice
#' @importFrom tidyr pivot_wider
#' @importFrom rlang sym
#' @importFrom ncdf4 nc_create ncvar_def ncdim_def ncvar_put ncatt_put nc_close
#' @importFrom tibble tibble
#' @examples
#' stations <- create_stations(n = 3)
#' sim <- simulate_climate_series(stations)
#' tmp <- tempfile(fileext = ".nc")
#' write_station_netcdf(sim, tmp)
write_station_netcdf <- function(df, out_nc = "simulated_station_climate.nc",
                                 fillvalue = -9999.0) {
  # Ensure required columns exist
  req <- c("Station","LON","LAT","Date","Avg.Tn","Avg.Tx","Sum.Rf","Avg.Rf")
  stopifnot(all(req %in% names(df)))
  # Normalize date format
  df <- df %>% mutate(Date = as.Date(Date))
  df2 <- df %>% arrange(Station, Date)
  # Helper function: reshape into matrix
  pivot_var <- function(var) {
    wide <- df2 %>%
      select(Station, Date, !!rlang::sym(var)) %>%
      tidyr::pivot_wider(names_from = Date, values_from = !!rlang::sym(var))
    mat <- as.matrix(wide[, -1])
    colnames(mat) <- format(as.Date(colnames(mat)), "%Y-%m-%d")
    rownames(mat) <- wide$Station
    list(mat = mat, stations = wide$Station, dates = as.Date(colnames(mat)))
  }
  # Pivot variables
  tmin <- pivot_var("Avg.Tn")
  tmax <- pivot_var("Avg.Tx")
  rsum <- pivot_var("Sum.Rf")
  ravg <- pivot_var("Avg.Rf")
  # Check time consistency
  if(!identical(tmin$dates, tmax$dates)) stop("Times misaligned (Tmin vs Tmax)")
  if(!identical(tmin$dates, rsum$dates)) stop("Times misaligned (Tmin vs Rain)")
  # Spatial data
  coords <- df2 %>%
    distinct(Station, .keep_all = TRUE) %>%
    select(Station, LON, LAT) %>%
    filter(Station %in% tmin$stations) %>%
    slice(match(tmin$stations, Station))
  # NetCDF dimensions
  nstations <- nrow(tmin$mat)
  time_vals <- as.numeric(tmin$dates - tmin$dates[1])
  station_dim <- ncdf4::ncdim_def("station", "", vals = 1:nstations, create_dimvar = FALSE)
  time_dim <- ncdf4::ncdim_def("time", paste0("days since ", tmin$dates[1]), vals = time_vals)
  # Define variables
  lon_var  <- ncdf4::ncvar_def("longitude", "degrees_east",  list(station_dim), fillvalue)
  lat_var  <- ncdf4::ncvar_def("latitude",  "degrees_north", list(station_dim), fillvalue)
  tmin_var <- ncdf4::ncvar_def("Tmin", "degC", list(station_dim, time_dim), fillvalue)
  tmax_var <- ncdf4::ncvar_def("Tmax", "degC", list(station_dim, time_dim), fillvalue)
  rsum_var <- ncdf4::ncvar_def("RainTotal", "mm", list(station_dim, time_dim), fillvalue)
  ravg_var <- ncdf4::ncvar_def("RainAvg", "mm", list(station_dim, time_dim), fillvalue)
  if (file.exists(out_nc)) file.remove(out_nc)
  # Create NetCDF safely
  nc <- ncdf4::nc_create(out_nc, list(lon_var, lat_var, tmin_var, tmax_var, rsum_var, ravg_var))
  # Always close on exit
  on.exit(ncdf4::nc_close(nc), add = TRUE)
  # Write data
  ncdf4::ncvar_put(nc, lon_var,  coords$LON)
  ncdf4::ncvar_put(nc, lat_var,  coords$LAT)
  ncdf4::ncvar_put(nc, tmin_var, tmin$mat)
  ncdf4::ncvar_put(nc, tmax_var, tmax$mat)
  ncdf4::ncvar_put(nc, rsum_var, rsum$mat)
  ncdf4::ncvar_put(nc, ravg_var, ravg$mat)
  # Metadata
  ncdf4::ncatt_put(nc, 0, "title", "Simulated Station Climate Data")
  ncdf4::ncatt_put(nc, 0, "history", paste("Created on", Sys.time()))
  invisible(out_nc)
}

---
title: "Getting Started with CDSim"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with CDSim}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 8,
  fig.height = 4,
  comment = "#>"
)
library(CDSim)
```
The CDSim package provides an easy workflow for simulating climate data such as temperature and rainfall across multiple synthetic weather stations. It is useful for testing models, teaching climate analysis, generating demo data, and creating datasets with controlled variability.

This vignette demonstrates:

 - how to create synthetic weather stations

 - how to generate multi-year climate time series

 - how to export results to CSV and NetCDF

 - how to perform a quick visualization

<h2 style="font-size:160%; font-weight:bold;"> Creating Weather Stations </h2>

The stations can be created either by:

  - loading from a CSV file,
  
  - accepting an existing data frame,
  
  - or auto-generating synthetic stations in a bounding box.
  
```{r}
library(CDSim)
```
We begin by generating a set of synthetic weather stations. The seed ensures
reproducibility.
```{r}
stations <- create_stations(n = 3, seed = 123)
stations
```
Each station typically contains:

  1. station name

  2. longitude

  3. latitude
  
<h2 style="font-size:160%; font-weight:bold;"> Simulating Climate Data </h2>

Once the stations are created, we can generate daily or monthly climate time
series using built-in stochastic models.
```{r}
sim <- simulate_climate_series(stations, start_year = 2019, end_year = 2024)
head(sim)
```
A typical simulated record includes:

  - date

  - min and max temperature

  - rainfall

  - station metadata
  
<h2 style="font-size:160%; font-weight:bold;"> Exporting Data to CSV and NetCDF </h2>

CDSim includes convenient exporters such as CSV and NetCDF for storing climate data. This makes it easier for packages such as ncdf4, terra, or stars to read the outputs.
```{r}
write_station_csv(sim, file = "climate_data.csv")
```
```{r}
write_station_netcdf(sim, out_nc = "climate_data.nc")
```
<h2 style="font-size:160%; font-weight:bold;"> Quick Visualization </h2>

To demonstrate a quick plot, here’s the maximum temperature series of the first station.
```{r}
plot_station_timeseries(sim,'Station_1', var = "Avg.Tx")
```
<h2 style="font-size:160%; font-weight:bold;"> Validation of Simulated Climate Data </h2>

To assess the physical realism and statistical consistency of the simulated climate data, CDSim provides a validation framework.

The function `validate_climate_internal()` performs a series of diagnostic checks on the simulated dataset, including:

- physical constraints (e.g., temperature bounds and rainfall non-negativity)
- distributional properties (e.g., variability and skewness)
- temporal dependence (autocorrelation)
- inter-variable relationships (e.g., rainfall–temperature coupling)
- trend behavior and seasonal patterns

```{r}
validation <- validate_climate_internal(sim)
validation
```
In addition, CDSim supports external validation against observed datasets using the function `validate_climate()`. This allows users to compare simulated data with real-world observations for further evaluation of model performance.

The validation framework is particularly useful in scenarios where observed data are unavailable, providing diagnostic assurance that the simulated outputs adhere to known climatological principles.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io.R
\name{write_station_netcdf}
\alias{write_station_netcdf}
\title{Write station NetCDF (station x time)
Exports a simulated climate station dataset to a NetCDF file.}
\usage{
write_station_netcdf(
  df,
  out_nc = "simulated_station_climate.nc",
  fillvalue = -9999
)
}
\arguments{
\item{df}{station x time long dataframe returned by simulate_climate_series()}

\item{out_nc}{Output NetCDF filename}

\item{fillvalue}{Value used for missing entries}
}
\value{
Returns the file path invisibly.
}
\description{
Write station NetCDF (station x time)
Exports a simulated climate station dataset to a NetCDF file.
}
\examples{
stations <- create_stations(n = 3)
sim <- simulate_climate_series(stations)
tmp <- tempfile(fileext = ".nc")
write_station_netcdf(sim, tmp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_climate.R
\name{simulate_climate_series}
\alias{simulate_climate_series}
\title{Simulate monthly climate time series for stations}
\usage{
simulate_climate_series(
  stations,
  start_year = 1996,
  end_year = 2025,
  seed = NULL,
  temp_trend_per_year = 0.02,
  rain_trend_per_year = -0.003,
  phi_temp = 0.85,
  sd = 0.4,
  Tmin_min = 18,
  Tmin_max = 30,
  Tmax_min = 24,
  Tmax_max = 42
)
}
\arguments{
\item{stations}{data.frame from create_stations() (Station, LON, LAT)}

\item{start_year}{integer (e.g., 1996)}

\item{end_year}{integer (e.g., 2025)}

\item{seed}{optional numeric seed}

\item{temp_trend_per_year}{temperature trend per year (°C/year warming)}

\item{rain_trend_per_year}{rain trend per year (slight drying trend)}

\item{phi_temp}{AR(1) persistence}

\item{sd}{standard deviation of the AR(1) innovation process controlling temperature variability}

\item{Tmin_min}{minimum value for minimum temperature}

\item{Tmin_max}{maximum value for minimum temperature}

\item{Tmax_min}{minimum value for maximum temperature}

\item{Tmax_max}{maximum value for maximum temperature}
}
\value{
A tidy data.frame with one row per station × month containing:
Station, LON, LAT, Year, Month, Date, Avg.Tn, Avg.Tx, Sum.Rf, Avg.Rf
}
\description{
Simulate monthly Tmin, Tmax, monthly total rainfall (Sum.Rf) and mean daily rainfall (Avg.Rf)
for each station across a year range.
}
\details{
This function generates synthetic monthly climate time series using a
stochastic, physically-informed modelling framework. Temperature is modeled
as a combination of deterministic seasonality, long-term trend, and
stochastic variability. The seasonal component is represented using a
sinusoidal function, while temporal persistence is introduced via an
autoregressive AR(1) process applied to the innovation term.

Minimum temperature (Avg.Tn) is simulated using a truncated normal
distribution to enforce physically realistic lower and upper bounds.
Maximum temperature (Avg.Tx) is generated using a gamma-distributed
perturbation applied to the mean temperature, producing an asymmetric
distribution consistent with observed climatological behavior.

Rainfall occurrence is modeled using a first-order Markov chain, allowing
for realistic wet–dry persistence. Conditional on occurrence, rainfall
intensity is drawn from a gamma distribution with seasonally varying mean.
A temporal trend term can be applied to represent long-term climatic changes
such as gradual drying or wetting.

To ensure physical consistency between variables, a coupling mechanism is
introduced whereby increased rainfall (proxy for cloud cover) reduces
maximum temperature through a linear cooling adjustment. This enforces
a negative dependence between precipitation and temperature consistent
with atmospheric energy balance principles.

Finally, a minimum diurnal temperature difference constraint is enforced
after rounding to guarantee that Avg.Tx > Avg.Tn at all time steps, while
preserving the statistical distribution of the simulated variables.

The default parameterization reflects typical tropical conditions for Ghana,
but all parameters are user-configurable, allowing adaptation to other
climatic regions.
The modelling approach follows established stochastic weather generation
principles while extending them with distributional asymmetry and
cross-variable coupling for improved physical realism.
}
\examples{
st <- create_stations(n = 3, seed = 1)
sim <- simulate_climate_series(st, 1996, 2025, seed = 42)
head(sim)
}
\seealso{
\code{\link[=write_station_csv]{write_station_csv()}}, \code{\link[=write_station_netcdf]{write_station_netcdf()}}
}
\concept{IO Functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{visualization}
\alias{visualization}
\title{Visualization Functions for Climate Data}
\description{
Visualization Functions for Climate Data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_stations.R
\name{create_stations}
\alias{create_stations}
\title{Create or load station metadata}
\usage{
create_stations(
  source = NULL,
  n = 10,
  bbox = c(-3.5, 1.5, 4.5, 11.5),
  seed = NULL
)
}
\arguments{
\item{source}{Path to CSV file OR a data.frame with Station/LON/LAT OR NULL (to generate synthetic).}

\item{n}{Integer number of stations to generate when source = NULL. Default 10.}

\item{bbox}{numeric vector c(min_lon, max_lon, min_lat, max_lat). Default ~ Ghana bounding box.}

\item{seed}{Optional numeric to make generation reproducible.}
}
\value{
A data.frame with columns Station, LON, LAT.
}
\description{
Create a station metadata table (Station, LON, LAT) either by:
\itemize{
\item loading from a CSV file,
\item accepting an existing data.frame,
\item or auto-generating synthetic stations in a bounding box.
}
}
\examples{
create_stations(n = 5, seed = 42)
create_stations(data.frame(Station="A", LON=0, LAT=5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io.R
\name{write_station_csv}
\alias{write_station_csv}
\title{Write station CSV
Exports a simulated climate station dataset to a CSV file.}
\usage{
write_station_csv(df, file = "simulated_station_climate.csv")
}
\arguments{
\item{df}{A dataframe returned by \code{simulate_climate_series()}.}

\item{file}{The output CSV filename.}
}
\value{
Returns the file path invisibly.
}
\description{
Write station CSV
Exports a simulated climate station dataset to a CSV file.
}
\examples{
stations <- create_stations(n = 3)
sim <- simulate_climate_series(stations)
tmp <- tempfile(fileext = ".csv")
write_station_csv(sim, tmp)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_station_timeseries}
\alias{plot_station_timeseries}
\title{Plot Station Time Series with Seasonal Detection}
\usage{
plot_station_timeseries(
  df,
  station,
  var = "Avg.Tn",
  smooth = TRUE,
  theme_dark = FALSE
)
}
\arguments{
\item{df}{A tidy dataset containing columns: \code{Station}, \code{Date}, \code{LAT}, and variables.}

\item{station}{Station name.}

\item{var}{Climate variable to plot.}

\item{smooth}{Add LOESS smoothing line.}

\item{theme_dark}{Use dark theme.}
}
\value{
A ggplot object.
}
\description{
Creates a time-series plot for climate variables with automatic hemisphere-based season detection.
}
\examples{
stations <- create_stations(n = 3)
sim <- simulate_climate_series(stations)
plot_station_timeseries(sim, station = "Station_1", var = "Avg.Tn")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CDSim-package.R
\docType{package}
\name{CDSim-package}
\alias{CDSim}
\alias{CDSim-package}
\title{CDSim: Climate Data Simulation Toolkit}
\description{
Tools for generating and exporting synthetic climate observation datasets.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ikemillar/CDSim}
  \item Report bugs at \url{https://github.com/ikemillar/CDSim/issues}
}

}
\author{
Isaac Osei and Acheampong Baafi-Adomako and Sivaparvathi Dusari
}
\keyword{climate}
\keyword{netcdf}
\keyword{rainfall}
\keyword{simulation}
\keyword{temperature}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validation.R
\name{validate_climate}
\alias{validate_climate}
\title{Validate simulated climate data against observations}
\usage{
validate_climate(sim, obs)
}
\arguments{
\item{sim}{Simulated climate data.frame}

\item{obs}{Observed climate data.frame}
}
\value{
A list containing validation metrics and test results
}
\description{
Performs statistical and physical validation of simulated climate data
against observed datasets, including distributional tests, mean comparison,
dependence structure, and temporal persistence.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validation.R
\name{validate_climate_internal}
\alias{validate_climate_internal}
\title{Internal validation of simulated climate data}
\usage{
validate_climate_internal(sim)
}
\arguments{
\item{sim}{Simulated climate data.frame}
}
\value{
A list of validation diagnostics
}
\description{
Evaluates physical plausibility and statistical properties of simulated
climate data in the absence of observational datasets. The function
assesses distributional characteristics, temporal persistence,
inter-variable relationships, and physical constraints.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/safe_name.R, R/utils.R
\name{safe_name}
\alias{safe_name}
\title{Make a safe filename}
\usage{
safe_name(x)

safe_name(x)
}
\arguments{
\item{x}{A character string to clean.}
}
\value{
A cleaned filename string.
}
\description{
Ensures file names contain only safe ASCII characters.
}
